library bottom_action_sheet;

import 'dart:io';
import 'package:flutter/cupertino.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';

class BottomActionSheet {
  static void displaySheet({
    required BuildContext context,
    required List<SheetAction> actions,
    Widget? title,
    Widget? message,
  }) {
    if (Platform.isIOS) {
      _showCupertinoActionSheet(context, title, message, actions);
    } else {
      _settingModalBottomSheet(context, title, message, actions);
    }
  }
}

void _showCupertinoActionSheet(BuildContext context, Widget? title,
    Widget? message, List<SheetAction> actions) {
  const noCancelOption = -1;
  var indexOfCancel = actions.lastIndexWhere((action) => action.isCancel);

  CupertinoActionSheet actionSheet;
  actionSheet = indexOfCancel == noCancelOption
      ? CupertinoActionSheet(
          title: title,
          message: message,
          actions: actions
              .where((action) => !action.isCancel)
              .map<Widget>(_cupertinoActionSheetActionFromAction)
              .toList())
      : CupertinoActionSheet(
          title: title,
          message: message,
          actions: actions
              .where((action) => !action.isCancel)
              .map<Widget>(_cupertinoActionSheetActionFromAction)
              .toList(),
          cancelButton:
              _cupertinoActionSheetActionFromAction(actions[indexOfCancel]));
  showCupertinoModalPopup(context: context, builder: (_) => actionSheet);
}

CupertinoActionSheetAction _cupertinoActionSheetActionFromAction(
        SheetAction action) =>
    CupertinoActionSheetAction(
      child: Text(action.text),
      onPressed: action.onPressed,
      isDefaultAction: action.defaultAction,
    );

ListTile _listTileFromAction(SheetAction action) => action.hasArrow
    ? ListTile(
        title: Text(action.text),
        onTap: action.onPressed,
        trailing: const Icon(Icons.keyboard_arrow_right),
      )
    : ListTile(
        title: Text(
          action.text,
          style: TextStyle(
              fontWeight:
                  action.defaultAction ? FontWeight.bold : FontWeight.normal),
        ),
        onTap: action.onPressed,
      );

void _settingModalBottomSheet(
    context, title, message, List<SheetAction> actions) {
  if (actions.isNotEmpty) {
    showModalBottomSheet(
        context: context,
        builder: (_) {
          const _lastItem = 1, _secondLastItem = 2;
          return Container(
            child: Column(
              mainAxisSize: MainAxisSize.min,
              crossAxisAlignment: CrossAxisAlignment.stretch,
              children: <Widget>[
                Padding(
                  padding: const EdgeInsets.only(left: 20),
                  child: title,
                ),
                Padding(
                  padding: const EdgeInsets.only(left: 20),
                  child: message,
                ),
                ListView.separated(
                    padding: const EdgeInsets.only(left: 20),
                    shrinkWrap: true,
                    itemCount: actions.length,
                    itemBuilder: (_, index) =>
                        _listTileFromAction(actions[index]),
                    separatorBuilder: (_, index) =>
                        (index == (actions.length - _secondLastItem) &&
                                actions[actions.length - _lastItem].isCancel)
                            ? const Divider()
                            : Container()),
              ],
            ),
          );
        });
  }
}

/// Data class for Actions in ActionSheet
class SheetAction {
  /// Text to display
  final String text;

  /// The function which will be called when the action is pressed
  final VoidCallback onPressed;

  /// Is this a default action - especially for iOS
  final bool defaultAction;

  /// This is a cancel option - especially for iOS
  final bool isCancel;

  /// on Android indicates that further options are next
  final bool hasArrow;

  /// Construction of an ActionSheetAction
  SheetAction({
    required this.text,
    required this.onPressed,
    this.defaultAction = false,
    this.isCancel = false,
    this.hasArrow = false,
  });
}
